@@ -1,5 +1,5 @@
 /*  floating_shape.h -- Low-ish level window creation and management.
- *  Copyright (C) 2008  Nick Gasson
+ *  Copyright (C) 2008-2020  Nick Gasson
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,16 +19,16 @@
 #define INC_FLOATING_SHAPE_H
 
 #include <gtk/gtk.h>
+#include <stdbool.h>
 
 /*
  * A widget type thing which used the XShape extension.
  */
 typedef struct {
    GtkWidget *window;
-   GtkWidget *image;
    GdkPixbuf *pixbuf;
-   GdkDrawable *mask_bitmap;
    int x, y, width, height;
+   bool has_rgba;
 } float_shape_t;
 
 float_shape_t *make_shape_from_pixbuf(GdkPixbuf *pixbuf);
